<?php

/**
 * @file
 * Module configuration form and related functions.
 */

/**
 * Function to get the list of portals from the api.
 *
 * @return array
 *   : List of portals.
 */
function get_portals() {
  $portals = [];
  $url = variable_get('amu_hal_url_ws', 'https://api.archives-ouvertes.fr/') . 'ref/instance';
  $result = drupal_http_request($url);
  $decoded = json_decode($result->data, TRUE);
  foreach ($decoded['response']['docs'] as $value) {
    $portals[$value['code']] = $value['name'];
  }
  return $portals;
}

/**
 * Module configuration form.
 *
 * @param array $form
 *   : Form.
 * @param array $form_state
 *   : Form state.
 *
 * @return array
 *   : Form.
 */
function form_admin_hal(array $form, array &$form_state) {
  $portals = get_portals();
  $amu_hal_url_ws = variable_get('amu_hal_url_ws', 'https://api.archives-ouvertes.fr/');
  $hal_id_machine_name = variable_get('hal_id_machine_name', 'field_identifiant_hal');
  $node_ref = variable_get('node_ref', 'liste_diffusion');
  $team_machine_name = variable_get('team_machine_name', 'field_liste_souscrite');
  $portal = variable_get('portal');
  $publications_title = variable_get('publications_title', 'Publications HAL');
  $show_user_publications = variable_get('show_user_publications', '');
  $field_nom_machine_name = variable_get('field_nom_machine_name', 'field_nom');
  $field_prenom_machine_name = variable_get('field_prenom_machine_name', 'field_prenomprofil');

  $form['api_url'] = [
    '#type' => 'textfield',
    '#title' => t('URL HAL'),
    '#default_value' => $amu_hal_url_ws,
    '#required' => TRUE,
    '#disabled' => TRUE,
  ];

  $form['portals'] = [
    '#type' => 'select',
    '#title' => t('Portail voulu'),
    '#default_value' => $portal,
    '#options' => $portals,
  ];

  $form['show_user_publications'] = [
    '#type' => 'checkboxes',
    '#title' => t('Afficher la tabulation publications pour les utilisateurs'),
    '#default_value' => $show_user_publications,
    '#options' => [1 => "Afficher"],
  ];

  $form['publications_title'] = [
    '#type' => 'textfield',
    '#title' => t('Titre des publications dans la page utilisateur'),
    '#description' => t('ex: Publications HAL'),
    '#default_value' => $publications_title,
    '#required' => TRUE,
  ];

  $form['hal_id_machine_name'] = [
    '#type' => 'textfield',
    '#title' => t("Nom machine de l'identifiant HAL dans la BDD"),
    '#description' => t('ex: field_identifiant_hal'),
    '#default_value' => $hal_id_machine_name,
    '#required' => TRUE,
  ];

  $form['field_nom_machine_name'] = [
    '#type' => 'textfield',
    '#title' => t("Nom machine du champ nom de l'utilisateur dans la BDD"),
    '#description' => t('ex: field_nom'),
    '#default_value' => $field_nom_machine_name,
    '#required' => TRUE,
  ];

  $form['field_prenom_machine_name'] = [
    '#type' => 'textfield',
    '#title' => t("Nom machine du champ prénom de l'utilisateur dans la BDD"),
    '#description' => t('ex: field_prenomprofil'),
    '#default_value' => $field_prenom_machine_name,
    '#required' => TRUE,
  ];

  $form['node_ref'] = [
    '#type' => 'textfield',
    '#title' => t("Nom machine du type de contenu duquel sont crées les équipes d'utilisateurs dans la BDD"),
    '#description' => t('ex: liste_diffusion'),
    '#default_value' => $node_ref,
    '#required' => TRUE,
  ];

  $form['team_machine_name'] = [
    '#type' => 'textfield',
    '#title' => t("Nom machine du champ équipe de l'utilisateur dans la BDD"),
    '#description' => t('ex: field_liste_souscrite'),
    '#default_value' => $team_machine_name,
    '#required' => TRUE,
  ];

  $form['submit'] = [
    '#type' => 'submit',
    '#value' => t('Enregistrer'),
    '#submit' => [
      '_amu_hal_update_submit',
    ],
  ];

  if (!empty($form_state['result'])) {
    $form['result'] = [
      '#markup' => render($form_state['result']),
    ];
  }

  return $form;
}

/**
 * Module configuration form submit handler.
 *
 * @param array $form
 *   : Form.
 * @param array $form_state
 *   : Form state.
 */
function _amu_hal_update_submit(array $form, array &$form_state) {
  $form_state['rebuild'] = TRUE;

  $show_user_publications = variable_get('show_user_publications', '');
  try {
    variable_set('amu_hal_url_ws', $form_state['values']['api_url']);
    variable_set('hal_id_machine_name', $form_state['values']['hal_id_machine_name']);
    variable_set('node_ref', $form_state['values']['node_ref']);
    variable_set('show_user_publications', $form_state['values']['show_user_publications']);
    variable_set('team_machine_name', $form_state['values']['team_machine_name']);
    variable_set('portal', $form_state['values']['portals']);
    variable_set('publications_title', $form_state['values']['publications_title']);
    variable_set('field_nom_machine_name', $form_state['values']['field_nom_machine_name']);
    variable_set('field_prenom_machine_name', $form_state['values']['field_prenom_machine_name']);

    $form_state['result'] = [
      '#markup' => '<h3>Modifications enregistrées avec succès</h3>',
    ];
  }
  catch (Exception $e) {
    drupal_set_message($e, "error");
  }
  if ($show_user_publications !== $form_state['values']['show_user_publications'][1]) {
    drupal_flush_all_caches();
  }
}
