<?php

/**
 * @file
 * All export related functions and form.
 */

/**
 * BibTex and rtf export form.
 *
 * @param array $form
 *   : Form.
 * @param array $form_state
 *   : Form state.
 *
 * @return array
 *   : Form.
 */
function _amu_hal_export_form(array $form, array &$form_state) {
  $form['publications-ids'] = [
    '#type' => 'hidden',
  ];

  $form['export-bibtex'] = [
    '#type' => 'submit',
    '#value' => t('Export BibTex'),
    '#id' => 'export-bibtex',
    '#submit' => [
      '_amu_hal_bibtex_submit',
    ],
  ];

  $form['export-rtf'] = [
    '#type' => 'submit',
    '#value' => t('Export RTF'),
    '#id' => 'export-rtf',
    '#submit' => [
      '_amu_hal_rtf_submit',
    ],
  ];

  return $form;
}

/**
 * Rtf button submit handler.
 *
 * @param array $form
 *   : Form.
 * @param array $form_state
 *   : Form state.
 */
function _amu_hal_rtf_submit(array $form, array &$form_state) {
  $_SESSION['amu-hal-publication-ids'] = $form_state['values']['publications-ids'];
  drupal_goto('rtf');
}

/**
 * Rtf mass export function.
 */
function _amu_hal_rtf_session() {
  $doc = $_SESSION['amu-hal-publication-ids'];
  unset($_SESSION['amu-hal-publication-ids']);
  $docs = explode("&", $doc);
  $url = _amu_hal_doc_url_generator($docs, 'json');
  $data = _amu_hal_api_request($url);
  $dataArr = [];
  foreach ($data->response->docs as $value) {
    $dataArr[] = (array) $value;
  }
  _amu_hal_rtf_export($dataArr);
}

/**
 * Generates rtf file.
 *
 * @param string $doc_id
 *   : Doc IDs concatenation.
 */
function _amu_hal_rtf($doc_id) {
  $doc_id = explode("&", $doc_id);
  $url = _amu_hal_doc_url_generator($doc_id, 'json');
  $data = _amu_hal_api_request($url);
  $dataArr = [];
  foreach ($data->response->docs as $value) {
    $dataArr[] = (array) $value;
  }
  _amu_hal_rtf_export($dataArr);
}

/**
 * Populates and downloads the rtf files.
 *
 * @param array $docs
 *   : Doc IDs.
 */
function _amu_hal_rtf_export(array $docs) {
  $loc = drupal_get_path('module', 'amu_hal') . "/includes/sample.rtf";
  $sample_loc = drupal_get_path('module', 'amu_hal') . "/includes/text.rtf";
  $new_rtf = _amu_hal_populate_rtf($docs, $loc, $sample_loc);
  $fr = fopen('sesstim.rtf', 'w');
  fwrite($fr, $new_rtf);
  fclose($fr);
  header("Content-type: application/rtf");
  header("Content-disposition: inline;      filename=sesstim.rtf");
  header("Content-length: " . strlen($new_rtf));
  echo $new_rtf;
}

/**
 * Used top pupoulate the sample rtf files with the publication data.
 *
 * @param array $docs
 *   : Doc values.
 * @param string $doc_file
 *   : The sample.rtf file location.
 * @param string $sample_file
 *   : The text.rtf file location.
 *
 * @return string
 *   : Rtf file as string.
 */
function _amu_hal_populate_rtf(array $docs, $doc_file, $sample_file) {
  $keys = [
    '%%authLastNameFirstName_s%%',
    '%%title_s%%',
    '%%uri_s%%',
    '%%journalTitle_s%%',
    '%%producedDateY_i%%',
    '%%volume_s%%',
    '%%issue_s%%',
    '%%page_s%%',
    '%%publisherLink_s%%',
  ];
  $replacements = array(
    '\\' => "\\\\",
    '{'  => "\{",
    '}'  => "\}",
  );

  $document = file_get_contents($doc_file);
  if (!$document) {
    return FALSE;
  }
  foreach ($docs as $vars) {
    $document .= file_get_contents($sample_file);
    if (!$document) {
      return FALSE;
    }
    foreach ($vars as $key => $value) {
      $search = "%%" . $key . "%%";

      switch ($key) {
        case 'authLastNameFirstName_s':
          $concat = "";
          foreach ($value as $k => $v) {
            if ($k == count($value) - 1) {
              $concat .= $v;
            }
            else {
              $concat .= $v . ", ";
            }
          }
          $value = $concat;
          break;

        case 'issue_s':
          if (!$vars['page_s']) {
            $value = "(" . $value[0] . ").";
          }
          else {
            $value = "(" . $value[0] . "):";
          }

          break;

        case 'title_s':
          $value = $value[0];
          if ((substr($value, -1) !== ".") && (substr($value, -1) !== "!") && (substr($value, -1) !== "?")) {
            $value .= ".";
          }
          break;

        case 'publisherLink_s':
          $value = " " . $value[0];
          break;

        case 'producedDateY_i':
          if (!$vars['volume_s'] && !$vars['issue_s'] && !$vars['page_s']) {
            $value = " " . $value . ".";
          }
          else {
            $value = " " . $value . ";";
          }
          break;

        case 'journalTitle_s':
        case 'page_s':
          $value .= ".";
          break;

        case 'volume_s':
          if (!$vars['issue_s'] && !$vars['page_s']) {
            $value .= ".";
          }
          elseif (!$vars['issue_s']) {
            $value .= ":";
          }
          break;
      }
      $chars = split($value);
      foreach ($chars as $char) {
        if (mb_ord($char) == 160) {
          $value = mb_ereg_replace($char, chr(32), $value);
        }
        if (mb_ord($char) > 833 && mb_ord($char) < 1010) {
          $value = mb_ereg_replace($char, '\\u' . mb_ord($char) . '\\\'3f', $value);
        }
      }
      $value = mb_convert_encoding($value, "Windows-1252", "UTF-8");
      foreach ($replacements as $orig => $replace) {
        $value = str_replace($orig, $replace, $value);
      }
      $document = str_replace($search, $value, $document);
      $document = str_replace("\\\\", '\\', $document);
      $document = str_replace("�", " ", $document);
    }
    $document = str_replace($keys, "", $document);
  }
  $document .= "}";
  return $document;
}

/**
 * BibTex button submit handler.
 *
 * @param array $form
 *   : Form.
 * @param array $form_state
 *   : Form state.
 */
function _amu_hal_bibtex_submit(array $form, array &$form_state) {
  $_SESSION['amu-hal-publication-ids'] = $form_state['values']['publications-ids'];
  drupal_goto('bibtex');
}

/**
 * BibTex mass export function.
 */
function _amu_hal_bibtex_session() {
  $doc = $_SESSION['amu-hal-publication-ids'];
  unset($_SESSION['amu-hal-publication-ids']);
  $docs = explode("&", $doc);
  $url = _amu_hal_doc_url_generator($docs, 'bibtex');
  $data = _amu_hal_bibtex_api_request($url);

  header("Content-type: text/plain");
  header("Content-Disposition: attachment; filename=BibTex.txt");

  print $data->data;
}

/**
 * Generate the BibTex document(s).
 *
 * @param string $doc_id
 *   : Doc IDs concatenation.
 */
function _amu_hal_bibtex($doc_id) {
  $doc_id = explode("&", $doc_id);
  $url = _amu_hal_doc_url_generator($doc_id, 'bibtex');
  $data = _amu_hal_bibtex_api_request($url);

  header("Content-type: text/plain");
  header("Content-Disposition: attachment; filename=BibTex.txt");

  print $data->data;
}

/**
 * Sends request to archives ouvertes api & returns the BibTex formatted result.
 *
 * @param string $url
 *   : Request url.
 *
 * @return string
 *   : BibTex formatted string.
 */
function _amu_hal_bibtex_api_request($url) {
  $result = drupal_http_request($url);
  return $result;
}

/**
 * Generates a document url depending on a specific output format type.
 *
 * @param array $doc_ids
 *   : Document IDs.
 * @param string $type
 *   : Format type.
 *
 * @return string
 *   : Url.
 */
function _amu_hal_doc_url_generator(array $doc_ids, $type) {
  $url = variable_get('amu_hal_url_ws') . 'search/' . variable_get('portal') . '/?q=halId_s:';
  $count = count($doc_ids);
  if ($count == 0) {
    return 0;
  }
  elseif ($count == 1) {
    $doc_id = $doc_ids[0];
    $url .= $doc_id;
  }
  else {
    $url .= "(";
    foreach ($doc_ids as $key => $doc_id) {
      if ($key == $count - 1) {
        $url .= $doc_id;
      }
      else {
        $url .= "$doc_id+OR+";
      }
    }
    $url .= ")";
  }
  $url .= "&wt=$type&fl=authLastNameFirstName_s,title_s,journalTitle_s,journalPublisher_s,producedDateY_i,volume_s,issue_s,page_s,publisherLink_s,uri_s,files_s&rows=2000";
  $url .= '&sort=producedDate_tdate+desc';
  return $url;
}

if (!function_exists('mb_ord')) {

  /**
   * PHP 7.2 mb_ord function implementation.
   *
   * @param string $char
   *   : Character.
   * @param string $encoding
   *   : Encoding.
   *
   * @return int
   *   : Returns a code point of character or FALSE on failure.
   */
  function mb_ord($char, $encoding = 'UTF-8') {
    if ($encoding === 'UCS-4BE') {
      list(, $ord) = (strlen($char) === 4) ? @unpack('N', $char) : @unpack('n', $char);
      return $ord;
    }
    else {
      return mb_ord(mb_convert_encoding($char, 'UCS-4BE', $encoding), 'UCS-4BE');
    }
  }

}

/**
 * Multi-byte character splitting .
 *
 * @param string $str
 *   : String.
 * @param int $len
 *   : Length.
 *
 * @return array
 *   : Array of characters.
 */
function split($str, $len = 1) {
  $arr    = [];
  $length = mb_strlen($str, 'UTF-8');
  for ($i = 0; $i < $length; $i += $len) {
    $arr[] = mb_substr($str, $i, $len, 'UTF-8');
  }
  return $arr;
}
